@@ -92,6 +92,13 @@ class Client:
         endpoint = "tasks/video/" + task_id
         return self.api.get_request(endpoint)
 
+    def find_video_classification_task(self, task_id: str) -> dict:
+        """
+        Find a signle video classification task.
+        """
+        endpoint = "tasks/video/classification/" + task_id
+        return self.api.get_request(endpoint)
+
     def find_video_task_by_name(self, project: str, task_name: str) -> dict:
         """
         Find a signle video task by name.
@@ -111,6 +118,7 @@ class Client:
         self,
         project: str,
         status: str = None,
+        external_status: str = None,
         tags: list = [],
         task_name: str = None,
         offset: int = None,
@@ -121,7 +129,8 @@ class Client:
         Returns up to 1000 at a time, to get more, set offset as the starting position to fetch.
 
         project is slug of your project. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag. (Optional)
         task_name is a task name. (Optional)
         offset is the starting position number to fetch. (Optional)
@@ -134,6 +143,8 @@ class Client:
         params = {"project": project}
         if status:
             params["status"] = status
+        if external_status:
+            params["externalStatus"] = external_status
         if tags:
             params["tags"] = tags
         if task_name:
@@ -148,6 +159,7 @@ class Client:
         self,
         project: str,
         status: str = None,
+        external_status: str = None,
         tags: list = [],
         task_name: str = None,
         offset: int = None,
@@ -158,7 +170,8 @@ class Client:
         Returns up to 1000 at a time, to get more, set offset as the starting position to fetch.
 
         project is slug of your project. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag. (Optional)
         offset is the starting position number to fetch. (Optional)
         limit is the max number to fetch. (Optional)
@@ -167,6 +180,8 @@ class Client:
         params = {"project": project}
         if status:
             params["status"] = status
+        if external_status:
+            params["externalStatus"] = external_status
         if tags:
             params["tags"] = tags
         if task_name:
@@ -181,6 +196,7 @@ class Client:
         self,
         project: str,
         status: str = None,
+        external_status: str = None,
         tags: list = [],
         task_name: str = None,
         offset: int = None,
@@ -191,7 +207,8 @@ class Client:
         Returns up to 10 at a time, to get more, set offset as the starting position to fetch.
 
         project is slug of your project. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag. (Optional)
         offset is the starting position number to fetch. (Optional)
         limit is the max number to fetch. (Optional)
@@ -203,6 +220,8 @@ class Client:
         params = {"project": project}
         if status:
             params["status"] = status
+        if external_status:
+            params["externalStatus"] = external_status
         if tags:
             params["tags"] = tags
         if task_name:
@@ -217,6 +236,7 @@ class Client:
         self,
         project: str,
         status: str = None,
+        external_status: str = None,
         tags: list = [],
         task_name: str = None,
         offset: int = None,
@@ -227,7 +247,8 @@ class Client:
         Returns up to 10 at a time, to get more, set offset as the starting position to fetch.
 
         project is slug of your project. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag. (Optional)
         task_name is a task name. (Optional)
         offset is the starting position number to fetch. (Optional)
@@ -240,6 +261,8 @@ class Client:
         params = {"project": project}
         if status:
             params["status"] = status
+        if external_status:
+            params["externalStatus"] = external_status
         if tags:
             params["tags"] = tags
         if task_name:
@@ -254,6 +277,7 @@ class Client:
         self,
         project: str,
         status: str = None,
+        external_status: str = None,
         tags: list = [],
         task_name: str = None,
         offset: int = None,
@@ -264,7 +288,8 @@ class Client:
         Returns up to 1000 at a time, to get more, set offset as the starting position to fetch.
 
         project is slug of your project. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag. (Optional)
         offset is the starting position number to fetch. (Optional)
         limit is the max number to fetch. (Optional)
@@ -273,6 +298,8 @@ class Client:
         params = {"project": project}
         if status:
             params["status"] = status
+        if external_status:
+            params["externalStatus"] = external_status
         if tags:
             params["tags"] = tags
         if task_name:
@@ -319,6 +346,7 @@ class Client:
         name: str,
         file_path: str,
         status: str = None,
+        external_status: str = None,
         annotations: list = [],
         tags: list = [],
     ) -> str:
@@ -328,7 +356,8 @@ class Client:
         project is slug of your project. (Required)
         name is an unique identifier of task in your project. (Required)
         file_path is a path to data. Supported extensions are png, jpg, jpeg. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
@@ -340,6 +369,8 @@ class Client:
         payload = {"project": project, "name": name, "file": file}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if annotations:
             for annotation in annotations:
                 annotation["content"] = name
@@ -354,6 +385,7 @@ class Client:
         name: str,
         file_path: str,
         status: str = None,
+        external_status: str = None,
         attributes: list = [],
         tags: list = [],
     ) -> str:
@@ -363,7 +395,8 @@ class Client:
         project is slug of your project. (Required)
         name is an unique identifier of task in your project. (Required)
         file_path is a path to data. Supported extensions are png, jpg, jpeg. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
@@ -375,6 +408,8 @@ class Client:
         payload = {"project": project, "name": name, "file": file}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if attributes:
             payload["attributes"] = attributes
         if tags:
@@ -387,6 +422,7 @@ class Client:
         name: str,
         folder_path: str,
         status: str = None,
+        external_status: str = None,
         annotations: list = [],
         tags: list = [],
     ) -> str:
@@ -396,7 +432,8 @@ class Client:
         project is slug of your project. (Required)
         name is an unique identifier of task in your project. (Required)
         folder_path is a path to data folder. Files should be under the folder. Nested folder structure is not supported. Supported extensions of files are png, jpg, jpeg. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
@@ -422,6 +459,8 @@ class Client:
         payload = {"project": project, "name": name, "contents": contents}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if annotations:
             payload["annotations"] = annotations
         if tags:
@@ -434,6 +473,7 @@ class Client:
         name: str,
         file_path: str,
         status: str = None,
+        external_status: str = None,
         annotations: list = [],
         tags: list = [],
     ) -> str:
@@ -442,8 +482,9 @@ class Client:
 
         project is slug of your project. (Required)
         name is an unique identifier of task in your project. (Required)
-        file_path is a path to data. Supported extensions are png, jpg, jpeg. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        file_path is a path to data. Supported extensions are mp4. (Required)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
@@ -455,6 +496,8 @@ class Client:
         payload = {"project": project, "name": name, "file": file}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if annotations:
             for annotation in annotations:
                 annotation["content"] = name
@@ -463,25 +506,65 @@ class Client:
             payload["tags"] = tags
         return self.api.post_request(endpoint, payload=payload)
 
+    def create_video_classification_task(
+        self,
+        project: str,
+        name: str,
+        file_path: str,
+        status: str = None,
+        external_status: str = None,
+        attributes: list = [],
+        tags: list = [],
+    ) -> str:
+        """
+        Create a single video classification task.
+
+        project is slug of your project. (Required)
+        name is an unique identifier of task in your project. (Required)
+        file_path is a path to data. Supported extensions are mp4. (Required)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
+        attributes is a list of attribute to be set in advance. (Optional)
+        tags is a list of tag to be set in advance. (Optional)
+        """
+        endpoint = "tasks/video/classification"
+        if not utils.is_video_supported_ext(file_path):
+            raise FastLabelInvalidException("Supported extensions are mp4.", 422)
+        file = utils.base64_encode(file_path)
+        payload = {"project": project, "name": name, "file": file}
+        if status:
+            payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
+        if attributes:
+            payload["attributes"] = attributes
+        if tags:
+            payload["tags"] = tags
+        return self.api.post_request(endpoint, payload=payload)
+
     # Task Update
 
     def update_task(
         self,
         task_id: str,
         status: str = None,
+        external_status: str = None,
         tags: list = [],
     ) -> str:
         """
         Update a single task.
 
         task_id is an id of the task. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag to be set. (Optional)
         """
         endpoint = "tasks/" + task_id
         payload = {}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if tags:
             payload["tags"] = tags
         return self.api.put_request(endpoint, payload=payload)
@@ -490,6 +573,7 @@ class Client:
         self,
         task_id: str,
         status: str = None,
+        external_status: str = None,
         attributes: list = [],
         tags: list = [],
     ) -> str:
@@ -497,7 +581,8 @@ class Client:
         Create a single image classification task.
 
         task_id is an id of the task. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
@@ -505,6 +590,8 @@ class Client:
         payload = {}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if attributes:
             payload["attributes"] = attributes
         if tags:
@@ -515,6 +602,7 @@ class Client:
         self,
         task_id: str,
         status: str = None,
+        external_status: str = None,
         attributes: list = [],
         tags: list = [],
     ) -> str:
@@ -522,7 +610,8 @@ class Client:
         Create a single video classification task.
 
         task_id is an id of the task. (Required)
-        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
+        external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
         """
@@ -530,6 +619,8 @@ class Client:
         payload = {}
         if status:
             payload["status"] = status
+        if external_status:
+            payload["externalStatus"] = external_status
         if attributes:
             payload["attributes"] = attributes
         if tags:
@@ -665,11 +756,14 @@ class Client:
             color = index if is_instance_segmentation else classes.index(annotation["value"]) + 1
             if annotation["type"] == AnnotationType.segmentation.value:
                 for region in annotation["points"]:
+                    count = 0
                     for points in region:
                         cv_draw_points = self.__get_cv_draw_points(points)
-                        cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
-                        # hollowd points are not supported
-                        break
+                        if count == 0:
+                            cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+                        else:
+                            cv2.fillPoly(image, [cv_draw_points], 0, lineType=cv2.LINE_8, shift=0)
+                        count += 1
             elif annotation["type"] == AnnotationType.polygon.value:
                 cv_draw_points = self.__get_cv_draw_points(annotation["points"])
                 cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
@@ -954,9 +1048,10 @@ class Client:
         type: str,
         name: str,
         slug: str,
-        is_bitmap: bool = False,
+        is_pixel: bool = False,
         job_size: int = 10,
-        use_annotation_service: bool = False
+        workflow: str = "two_step",
+        external_workflow: str = "two_step",
     ) -> str:
         """
         Create a project.
@@ -964,22 +1059,23 @@ class Client:
         type can be 'image_bbox', 'image_polygon', 'image_keypoint', 'image_line', 'image_segmentation', 'image_classification', 'image_all', 'multi_image_bbox', 'multi_image_polygon', 'multi_image_keypoint', 'multi_image_line', 'multi_image_segmentation', 'video_bbox', 'video_single_classification'. (Required)
         name is name of your project. (Required)
         slug is slug of your project. (Required)
-        is_bitmap is whether to be annotated by pixel. (Optional)
+        is_pixel is whether to annotate image with pixel level. (Optional)
         job_size is the number of tasks the annotator gets at one time. (Optional)
-        use_annotation_service is whether to request FastLabel to provide annotation service or not. (Optional)
+        workflow is the type of annotation wokflow. workflow can be 'two_step' or 'three_step' (Optional)
+        external_workflow is the type of external annotation wokflow. external_workflow can be 'two_step' or 'three_step' (Optional)
         """
         endpoint = "projects"
         payload = {
             "type": type,
             "name": name,
             "slug": slug,
+            "workflow": workflow,
+            "externalWorkflow": external_workflow,
         }
-        if is_bitmap:
-            payload["isBitmap"] = is_bitmap
+        if is_pixel:
+            payload["isPixel"] = is_pixel
         if job_size:
             payload["jobSize"] = job_size
-        if use_annotation_service:
-            payload["useAnnotationService"] = use_annotation_service
         return self.api.post_request(endpoint, payload=payload)
 
     def update_project(
@@ -988,6 +1084,8 @@ class Client:
         name: str = None,
         slug: str = None,
         job_size: int = None,
+        workflow: str = None,
+        external_workflow: str = None,
     ) -> str:
         """
         Update a project.
@@ -996,6 +1094,8 @@ class Client:
         name is name of your project. (Optional)
         slug is slug of your project. (Optional)
         job_size is the number of tasks the annotator gets at one time. (Optional)
+        workflow is the type of annotation wokflow. workflow can be 'two_step' or 'three_step' (Optional)
+        external_workflow is the type of external annotation wokflow. external_workflow can be 'two_step' or 'three_step' (Optional)
         """
         endpoint = "projects/" + project_id
         payload = {}
@@ -1005,6 +1105,10 @@ class Client:
             payload["slug"] = slug
         if job_size:
             payload["jobSize"] = job_size
+        if workflow:
+            payload["workflow"] = workflow
+        if external_workflow:
+            payload["externalWorkflow"] = external_workflow
         return self.api.put_request(endpoint, payload=payload)
 
     def delete_project(self, project_id: str) -> None:
